package ihm.runtimeIhm;

import java.awt.Font;
import java.awt.Point;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import javax.management.InvalidAttributeValueException;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;

import csp_runtime.CspThread;
import csp_runtime.DebugListener;
import csp_runtime.DebugObj;
import csp_runtime.ValidableListener;

public class DebugFrame extends JInternalFrame {
	
	public static final Font DEFAULT_FONT = new Font ("courier new", 1, 12);
	
	private static final int DEFAULT_TAB_SIZE = 4;
	
	private Map<CspThread, Vector<Object>> highlighters = new HashMap<CspThread, Vector<Object>>();
	private DebugWindow debugWindow;
	
	private JScrollPane scroll;
	private JTextArea   area;
	
	private Point first;

	private boolean modeGeneral = false;
	private Collection<ValidableListener> listeners = Collections.synchronizedCollection(new Vector<ValidableListener> ());
	
	public DebugFrame (DebugWindow debugWindow) {
		super ("", true, true, true, true);
		
		this.debugWindow = debugWindow;

		initUI ();
		
		area.setText(debugWindow.getCspText());
		
		first = new Point (1, 1);
	}
	
	private void initUI() {
		area = new JTextArea();
		area.setHighlighter(new MyHighlighter());
		area.setFont (DEFAULT_FONT);
		area.setTabSize(DEFAULT_TAB_SIZE);
		scroll = new JScrollPane (area);
		
		setModeGeneral (false);
		
		setContentPane (scroll);
		area.setEditable(false);

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent e) {
				if (!modeGeneral) {
					closeListeners ();
				}
				setVisible (false);
			}
		});		
	}

	private void closeListeners () {
		for (ValidableListener listener : listeners)
			listener.valid.set(false);
		setVisible (false);
	}
	
	public DebugFrame (DebugWindow debugWindow, CspThread ... threads) throws InvalidAttributeValueException {
		super ("", true, true, true, true);

		this.debugWindow = debugWindow;

		initUI ();
		
		if (threads.length==0) throw new InvalidAttributeValueException ("Au moins un thread doit tre rattach  la fenetre, ou utiliser l'autre constructeur");
		
		Class theClass = threads[0].getClass();		
		
		for (CspThread thread : threads)
			if (!thread.getClass().equals(theClass))
				throw new InvalidAttributeValueException ("Les threads a associer  une fenetre de debug doivent tre de mme type!");

		first = threads[0].getFirstPoint();

		area.setText(debugWindow.getCspText(threads[0].getFirstPoint(), threads[0].getLastPoint()));
		
		for (CspThread thread : threads)
			addThread (thread);
		
	}
	
	private synchronized void manageDebug (CspThread source, DebugObj [] debugObjs) {
		try {
			Vector<Object> highs = highlighters.get(source);
			for (Object hObj : highs) {
				area.getHighlighter().removeHighlight(hObj);
			}
			highs.removeAllElements();
			if (debugObjs!=null) {
				for (DebugObj dObj : debugObjs) {
					try {
						int off1 = area.getLineStartOffset(dObj.first.y - first.y) + ((dObj.first.y!=first.y)?(dObj.first.x-1):(dObj.first.x - first.x));
						int off2 = area.getLineStartOffset(dObj.last.y - first.y) + ((dObj.last.y!=first.y)?(dObj.last.x):(dObj.last.x - first.x));
						
						highs.add(area.getHighlighter().addHighlight(off1, off2, new DefaultHighlighter.DefaultHighlightPainter (debugWindow.getDebugColor())));
					} catch (BadLocationException e) {
					}
									
				}
			}
			//scroll.repaint();
			debugWindow.getDesktop().repaint();
		} catch (Exception e) {
			
		}
	}
	
	public void addThread (final CspThread thread) {
		highlighters.put(thread, new Vector<Object>());
		
		if (!modeGeneral) setTitle (getTitle () + " " + thread.getCompleteName());
		
		manageDebug(thread, thread.getLastDebugObjs());
		
		DebugListener listener = new DebugListener() {
			public void debug(DebugObj... debugObjs) {
				if (valid.get()) {
					manageDebug (thread, debugObjs);
				}
			}
		};
			
		thread.addDebugListener(listener);
		
		listeners.add (listener);
		
	}

	public void setModeGeneral(boolean b) {
		modeGeneral  = b;
		
		if (b) {
			setTitle ("Debug gnral");
		} else {
			setTitle ("Debug ");
			for (CspThread thread : highlighters.keySet())
				setTitle (getTitle() + " " + thread.getCompleteName());
		}
	}
	
}
