package ihm.runtimeIhm;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.management.InvalidAttributeValueException;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import util.GBFactory;
import csp_runtime.CspThread;
import csp_runtime.DebugListener;
import csp_runtime.DebugObj;
import csp_runtime.EndThreadListener;
import csp_runtime.NewThreadListener;

/**
 * DebugManagementPanel est un panel permettant a l'utilisateur
 * de grer le debuggage.
 * 
 * @author Benjamin LE BOZEC
 */
public class DebugManagementPanel extends JPanel {

	private static final int defaultBoutonAnchor = GridBagConstraints.WEST;
	private static final Insets defaultBoutonInsets = new Insets(5,5,5,5);
	private static final Font DEFAULT_LABEL_STATE = new Font ("", 0, 12);
	
	private DebugWindow debugWindow;
	
	private JLabel labelState = new JLabel ("Prt  tre excut", JLabel.CENTER); 
	private JButton boutonLayout = new JButton ("layout");
	private JButton boutonColor  = new JButton ("color");
	private JButton boutonRun = new JButton ("run");
	private JButton boutonPause = new JButton ("pause");
	private JButton boutonTerminate = new JButton ("terminate");
	private JButton boutonPasAPas = new JButton ("pas  pas");
	private JButton boutonTempo  = new JButton ("temporiser");
	private JCheckBox cbGrouper = new JCheckBox ("grouper");
	private JListThread listThread = new JListThread (debugWindow);
	
	private ConsoleFrame consoleGenerale = new ConsoleFrame ();
	private DebugFrame   debugGeneral;
	
	private boolean alive = false;
	
	private Sync sync = new Sync ();
	private Temporisateur tempo;
	private boolean closeAfterEnd = false;
	
	public DebugManagementPanel(DebugWindow window) {
		this.debugWindow = window;
		
		setLayout (new GridBagLayout());
		
		/*
		 * Console et dbug gnrale
		 */
		debugWindow.getDesktop().addFrame(consoleGenerale);
		consoleGenerale.setModeGeneral(true);
		debugGeneral = new DebugFrame (debugWindow);
		debugWindow.getDesktop().addFrame (debugGeneral);
		debugGeneral.setModeGeneral (true);
		
		/*
		 * config du labelState
		 */
		labelState.setFont (DEFAULT_LABEL_STATE);
		
		/*
		 * config des boutons
		 */
		boutonLayout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				debugWindow.getDesktop().layoutFrames();
			}		
		});
		boutonRun.addActionListener (new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				run ();
			}
		});
		boutonPause.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				pause ();
			}
		});
		boutonPause.setEnabled (false);
		boutonTerminate.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				terminate ();
			}
		});
		boutonTerminate.setEnabled(false);
		boutonPasAPas.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				pasAPas();
			}
		});
		boutonTempo.addActionListener (new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				temporiser ();
			}
		});
		
		/*
		 * Config de la liste des thread
		 */
		listThread.setPreferredSize(new Dimension(100, 300));
		
		/*
		 * Config du bouton de couleur
		 */
		boutonColor.addActionListener (new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
				chooseColor ();
			}
		
		});
		
		/*
		 * config des boutons de cration de mini-fenetres
		 */
		JButton boutonNewConsole = new JButton ("console");
		boutonNewConsole.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newConsole ();
			}

		});
		JButton boutonNewDebug   = new JButton ("debug");
		boutonNewDebug.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				newDebug ();
			}
		});
		JButton boutonConsoleGenerale = new JButton ("Console gnrale");
		boutonConsoleGenerale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!consoleGenerale.isVisible()) {
					debugWindow.getDesktop().putIntoDesktop(consoleGenerale);
				}
			}
		});
		JButton boutonDebugGenerale = new JButton ("Debug gnral");
		boutonDebugGenerale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!debugGeneral.isVisible()) {
					debugWindow.getDesktop().putIntoDesktop(debugGeneral);
				}
			}
		});
		
		/*
		 * disposition gnrale des lments
		 */
		JPanel panelProc = new JPanel (new GridBagLayout());
		panelProc.add (labelState,
				GBFactory.getCst(0, 0, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelProc.add (boutonRun, 
				GBFactory.getCst(0, 1, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelProc.add (boutonPause, GBFactory.getCst(0, 2, 1, 1, 100, 0,  defaultBoutonInsets,defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelProc.add (boutonPasAPas, GBFactory.getCst(0, 3, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelProc.add (boutonTempo, GBFactory.getCst(0, 4, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelProc.add (boutonTerminate, GBFactory.getCst(0, 5, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		add(panelProc, 
				GBFactory.getCst(0, 0, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		JPanel panelBidon = new JPanel ();
		panelBidon.setMinimumSize(new Dimension(100, 20));
		add(panelBidon,
				GBFactory.getCst(0, 1, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		JPanel panelAffichage = new JPanel (new GridBagLayout());
		panelAffichage.setBorder(				
				BorderFactory.createTitledBorder(
						BorderFactory.createRaisedBevelBorder(),
						"Affichage"));
		panelAffichage.add (new JLabel("Threads en cours : "),
				GBFactory.getCst(0, 0, 1, 1, 0, 0, GridBagConstraints.NONE));
		panelAffichage.add (listThread, 
				GBFactory.getCst(0, 1, 1, 1, 100, 100, GridBagConstraints.BOTH));
		panelAffichage.add (cbGrouper,
				GBFactory.getCst(0, 2, 1, 1, 100, 0, GridBagConstraints.HORIZONTAL));
		panelAffichage.add (boutonNewConsole, 
				GBFactory.getCst(0, 3, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelAffichage.add (boutonNewDebug, 
				GBFactory.getCst(0, 4, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelAffichage.add (boutonConsoleGenerale, 
				GBFactory.getCst(0, 5, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelAffichage.add (boutonDebugGenerale, 
				GBFactory.getCst(0, 6, 1, 1, 100, 0, defaultBoutonInsets, defaultBoutonAnchor, GridBagConstraints.HORIZONTAL));
		panelBidon = new JPanel (new GridLayout(0,2));
		panelBidon.add (boutonLayout);
		panelBidon.add (boutonColor);
		panelAffichage.add (panelBidon, 
				GBFactory.getCst(0, 7, 1, 1, 100, 0, defaultBoutonInsets, GridBagConstraints.CENTER, GridBagConstraints.NONE));
		add(panelAffichage,
			GBFactory.getCst(0, 2, 1, 1, 100, 100, GridBagConstraints.BOTH));
				
		/*
		 * config du globalThreadFrame 
		 */
		CspThread.addNewThreadListener(new NewThreadListener () {
			public void newThread(CspThread nouv) {
				consoleGenerale.addThread (nouv);
				debugGeneral.addThread(nouv);
				nouv.addBlockListener(new DebugListener() {
					public void debug(DebugObj ... debugObjs) {
						sync.blockMe();
					}
				});
			}
		});
		
		CspThread.addEndThreadListener(new EndThreadListener() {
		
			public void endThread(CspThread thread) {
				if (thread.getClass().getName().endsWith(".MAIN")) {
					terminate();
					alive = false;
					labelState.setText ("Termin");
					if (closeAfterEnd) debugWindow.close ();
				}
			}
		
		});
		
		/*
		 * config gnrale
		 */
		setPreferredSize(new Dimension(170, 300));
	}

	public void run () {
		if (tempo!=null) { tempo.stop (); tempo = null; }
		if (!alive) {
			sync.setMode (Sync.Mode.RUN);
			debugWindow.getThread().start();
			alive = true;
		} else {
			sync.setMode (Sync.Mode.RUN);
			sync.hop();
		}
		labelState.setText("Temps rel");
		boutonRun.setEnabled(false);
		boutonPause.setEnabled(true);
		boutonTerminate.setEnabled (true);
	}
	
	public void pause () {
		if (tempo!=null) { tempo.stop (); tempo = null; }
		sync.setMode (Sync.Mode.EN_ATTENTE);
		labelState.setText("Pause");
		boutonPause.setEnabled (false);
		boutonRun.setEnabled(true);
	}
	
	public void terminate () {
		if (alive) {
			labelState.setText ("Terminaison...");
			sync.setMode (Sync.Mode.RUN);
			sync.hop();
			CspThread.terminateAll();
			boutonTerminate.setEnabled (false);
			boutonPause.setEnabled(false);
			boutonRun.setEnabled(false);
			boutonPasAPas.setEnabled(false);
			boutonTempo.setEnabled(false);
		}
	}
	
	public void pasAPas () {
		if (tempo!=null) { tempo.stop (); tempo = null; }
		if (!alive) {
			sync.setMode (Sync.Mode.EN_ATTENTE);
			debugWindow.getThread().start();
			alive = true;
		} else {
			sync.setMode (Sync.Mode.EN_ATTENTE);
			sync.hop();
		}
		labelState.setText ("Pas  pas");
		boutonTerminate.setEnabled (true);
		boutonPause.setEnabled(false);
		boutonRun.setEnabled(true);
	}

	private void temporiser () {
		String timing = JOptionPane.showInputDialog(debugWindow, "Donnez un timing (en ms) :");
		if (timing == null) return;
		if (timing.matches("[0-9]+")) {
			pasAPas ();
			tempo = new Temporisateur (sync, Integer.valueOf(timing));
			labelState.setText ("temporis");
			boutonPause.setEnabled(true);
		} else {
			JOptionPane.showMessageDialog (debugWindow,"Timing invalide : veuillez saisir un entier");
		}
		
	}

	private void newConsole() {
		CspThread [] selected = listThread.getSelectedThreads();
		
		if (selected.length==0) {
			JOptionPane.showMessageDialog(debugWindow, "Aucun thread selectionn");
			return;
		}
		
		if (selected.length==1) {
			ConsoleFrame console = new ConsoleFrame ();
			console.setThread(selected[0]);
			debugWindow.getDesktop().addFrame (console);
			return;
		}
		
		if (cbGrouper.isSelected()) {
			ConsoleFrame console = new ConsoleFrame ();
			console.setThreads (selected);
			debugWindow.getDesktop().addFrame (console);
		} else {
			for (CspThread thread : selected) {
				ConsoleFrame console = new ConsoleFrame ();
				console.setThread(thread);
				debugWindow.getDesktop().addFrame(console);
			}
		}

	}
	
	
	private void chooseColor () {
		Color color = JColorChooser.showDialog(debugWindow, "Couleur de dbuggage", debugWindow.getDebugColor());
		if (color!=null)
			debugWindow.setDebugColor (color);
	}
	
	private void newDebug () {
		CspThread [] selected = listThread.getSelectedThreads();
		
		if (selected.length==0) {
			JOptionPane.showMessageDialog(debugWindow, "Aucun thread selectionn");
			return;
		}
		
		if (cbGrouper.isSelected()) {
			DebugFrame debug;
			try {
				debug = new DebugFrame (debugWindow, selected);
			} catch (InvalidAttributeValueException e) {
				JOptionPane.showMessageDialog(debugWindow, e.getMessage());
				return;
			}
			debugWindow.getDesktop().addFrame (debug);
		} else {
			for (CspThread thread : selected) {
				DebugFrame debug;
				try {
					debug = new DebugFrame (debugWindow, thread);
				} catch (InvalidAttributeValueException e) {
					JOptionPane.showMessageDialog(debugWindow, e.getMessage());
					return;
				}
				debugWindow.getDesktop().addFrame(debug);
			}
		}
	}

	public boolean isAlive() {
		return alive;
	}

	public void setCloseAfterEnd(boolean b) {
		closeAfterEnd = b;
	}

}
