package ihm.runtimeIhm;


import ihm.compilateur.MainWindow;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import util.GBFactory;
import csp_runtime.CspThread;

/**
 * DebugWindow affiche une fenetre permettant  l'utilisateur de visualiser le 
 * droulement d'un programme CSP.
 * 
 * @author Benjamin LE BOZEC
 *
 */
public class DebugWindow extends JFrame {

	public static final int inset = 50;
	
	private CspThread thread;
	
	private ThreadDesktop desktop = new ThreadDesktop(this);
	private JPanel mainPanel = new JPanel ();
	private DebugManagementPanel management;
	private MainWindow main;	
	
	private String contentCspFile = "";
	private String [] linesCspFile;

	/**
	 * Constructeur 
	 * 
	 * @param main
	 * @param thread
	 */
	public DebugWindow (MainWindow main, CspThread thread) {
		this.main = main;
		this.thread = thread;		
		
		readFile (main.getCspFile());	
		
		management = new DebugManagementPanel (this);
		
		mainPanel.setLayout (new GridBagLayout());
		
		mainPanel.add(management,
				GBFactory.getCst(0, 0, 1, 1, 0, 100, GridBagConstraints.VERTICAL));
		
		mainPanel.add(desktop,
				GBFactory.getCst(1, 0, 1, 1, 100, 100, GridBagConstraints.BOTH));
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				if (management.isAlive()) {
					management.setCloseAfterEnd (true);
					management.terminate();
				} else
					close ();
			}
		});
		
		/*
		 * Config de la fenetre
		 */
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds (inset, inset, 
				(screen.width - inset*2), (screen.height - inset*2));
		setTitle (new File(main.getCspFile()).getName());
		setContentPane (mainPanel);
		
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		// on s'assure que rien ne sortira sur la sortie standard 
		CspThread.setSortieConsoleStd(false);
		setVisible (true);
	}

	public void close () {
		thread.clear();
		setVisible (false);
		main.setVisible (true);
	}
	
	private void readFile(String cspFile) {
		try {
			BufferedReader reader = new BufferedReader (new InputStreamReader(new FileInputStream (cspFile)));
			
			String line;
			Vector<String> lines = new Vector<String> ();
			contentCspFile = "";
			while ((line=reader.readLine())!=null) {
				contentCspFile += line + "\n";
				lines.add (line);
			}

			linesCspFile = new String [lines.size()];
			lines.toArray(linesCspFile);
			reader.close ();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(this, "Erreur d'e/s dans DebugWindow.readFile()");
		}
	}

	public CspThread getThread () { return thread; }
	
	public ThreadDesktop getDesktop () { return desktop; }

	public String getCspText(Point firstPoint, Point lastPoint) {
		String content = "";
		
		if (firstPoint.y != lastPoint.y) {
			content += linesCspFile[firstPoint.y-1].substring(firstPoint.x-1) + "\n";
			
			for (int i=firstPoint.y;i<lastPoint.y-1;i++)
				content += linesCspFile[i] + "\n";
			
			content += linesCspFile[lastPoint.y-1].substring(0, lastPoint.x);
		} else {
			content += linesCspFile[firstPoint.y-1].substring(firstPoint.x-1, lastPoint.x);
		}
			
		return content;
	}

	public String getCspText() {
		return contentCspFile;
	}

	public void setDebugColor (Color c) {
		main.setDebugColor (c);
	}
	
	public Color getDebugColor () {
		return main.getDebugColor ();
	}
}
