/*
 * Created on 25 fvr. 2005
 * 	
 */
package util.saveObject;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;

/**
 * <p>Cette classe permet la rcupration d'objet sauvegard  l'aide
 * de la classe OutputSaveObject.
 * 
 * <p>La mthode getNext permet de rcuperer les objets.
 * <p>Attention : les objets sont retourns dans l'ordre o ils ont t ajouts 
 * (cad dans l'ordre d'appel de la mthode "add" de OutputSaveObject).
 * <p>Par exemple, dans l'exemple donn dans "OutputSaveObject", les donnes seraient
 * rcuprable de la manire suivante :
 * <pre>
 * Class c1;
 * FileInputStream fis = new FileInputStream ("fichier.xml");
 * InputSaveObject sObj = new InputSaveObject (fis);
 * c1 = (Class1) sObj.getNext ("c1");
 * </pre>
 * 
 * @author Benjamin LE BOZEC
 */

public class InputSaveObject extends SaveObject  {

	private HashMap objectTable;
	
	/**
	 * Permet de reconstituer l'ensemble des images des objets enregistrs au
	 * format XML.
	 *  
	 * @param in Flux contenant les objets aux formats XML
	 * @throws SAXException Problme lors de la rcupration du fichier XML
	 * @throws ParserConfigurationException Problme de cration de l'objet SAXParser
	 * @throws IOException problme d'accs au flux "in"
	 */
	public InputSaveObject (InputStream in) 
	throws SaveObjectException {
		super ();
		objectTable = new HashMap (10);
		
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			SAXParser saxParser;
			saxParser = factory.newSAXParser();
			XmlHandler handler = new XmlHandler (this);
			saxParser.parse(in, handler);
		} catch (Exception e) {
			throw new SaveObjectException (e);
		}
			
		
	}
	
	/**
	 * Cr un InputSaveObject avec une table des objets dja cre 
	 * @param objectTable La table des objets
	 */
	private InputSaveObject (HashMap objectTable) {
		this.objectTable = objectTable;
	}
	
	/**
	 * Cration d'un nouvel lment contenant automatiquement un nouveau
	 * InputSaveObject.
	 * 
	 * Cette fonction est utilise par XmlHandler pour reconsistuer l'architecture
	 * des images.
	 * 
	 * @return l'lement cr
	 */
	protected ElemSaveObject createNewItem () {
		SaveObject sObj = new InputSaveObject (objectTable);
		ElemSaveObject item = new ElemSaveObject();
		item.value = sObj;
		data.addLast(item);
		return item;
	}

	/**
	 * Cration d'un nouvel lment vide
	 * 
	 * Cette fonction est utilise par XmlHandler pour reconsistuer l'architecture
	 * des images.
	 *
	 * @return L'lment cr
	 */
	protected ElemSaveObject createNewEmptyItem () {
		ElemSaveObject item = new ElemSaveObject();
		data.addLast(item);
		return item;
	}

	/**
	 * Permet de rcuprer un objet sauvegarder. Les objets sont retourns
	 * dans l'ordre o ils ont t ajouts dans le "OutputSaveObject".
	 * 
	 * @param name Nom de l'objet. Ce paramtre n'influe pas sur la valeur de retour
	 * mais permet juste de vrifier que l'objet qui suit est bien l'objet attendu.
	 * @return L'objet recr
	 * @throws SaveObjectException
	 * @throws EmptyObjectException si l'objet est vide (plus de donnes  rcuprer)
	 * @throws NameMismatchException si l'objet demand de correspond pas a l'objet suivant
	 */
	public Object getNext (String name) 
	throws SaveObjectException 
	{
		// s'il ne reste rien dans la liste des objets, on lance l'exception EmptyObjectException
		if (data.size()==0) throw new EmptyObjectException ();
		
		Object obj = null;
		Class c;
		ElemSaveObject item = (ElemSaveObject) data.getFirst();
		
		// Si le nom de l'objet qui va tre retourn ne correspondant pas au nom
		// donn en paramtre (name), alors l'exception NameMismatchException est leve
		if (!item.name.equals(name)) throw new NameMismatchException ();
		
		// rcupration de la classe de l'objet
		try {
			c = Class.forName(item.className);
		} catch (ClassNotFoundException e2) {
			throw new UnreachableClassException ("La classe " + item.className + " n'existe pas");
		}
		
		Constructor constructor;
		if (isPrimitive(c)) {
			// si l'objet est de type primitif (Integer, String, ...), on le cr
			// en lui donnant en paramtre item.value (<=> string)
			try {
				Class param [] = {String.class};
				constructor = c.getConstructor(param);
			} catch (SecurityException e1) {
				// la classe n'est pas publique
				throw new UnreachableClassException ("La classe " + item.className + " ne peut tre atteinte (public ?)");
			} catch (NoSuchMethodException e1) {
				// pas de constructeur prenant un objet "String" n'a t trouv
				throw new UnreachableClassException ("La classe " + item.className + " ne contient aucun constructeur prenant un objet String en paramtre");
			}
			
			try {
				Object initArgs [] = {item.value};
				obj = constructor.newInstance(initArgs);
			} catch (Exception e) {
				// probleme lors de la cration de l'objet
				throw new UnreachableClassException (e.toString());
			}
		} else {
			// si l'objet n'est pas de type primitif, on le cr a l'aide du 
		    // constructeur par dfaut de sa classe, puis on appelle 
			// sa mthode loadFrom
			obj =objectTable.get(item.reference); 
			if (obj==null) {
				// si l'objet n'a pas dja t cr, alors on le cr et on le rfrence
				try {
					obj = c.newInstance();
				} catch (Exception e) {
					// la classe n'a pas de constructeur pas dfaut visible
					throw new UnreachableClassException ("La classe " + c + " n'a aucun constructeur par dfaut visible (public)");
				}
				objectTable.put(item.reference, obj);
				((Savable) obj).loadFrom((InputSaveObject)item.value);
			}
		}
		
		if (obj!=null) data.removeFirst();
		/* on retire l'objet de la table des objets si plus aucun autre lment
		// ne le rfrence
		if (!findRef(item.reference)) 
			objectTable.remove(item.reference);*/
		return obj;
	}

	/**
	 * Permet de rcuperer un tableau
	 * @param name nom du tableau
	 * @return le tableau prcedemment stock
	 * @throws SaveObjectException
	 */
	public Object[] getNextTab (String name) 
	throws SaveObjectException {
		SavableArray sTab = (SavableArray) getNext (name);
		return sTab.getTab ();
	}
	
	/**
	 * Cette mthode parcours l'arborescence "data" afin de trouver d'ventuelle
	 * rfrences  "ref".
	 * 
	 * @param ref La rfrence  trouver
	 * @return VRAI si l'arborescence contient au moins une fois "ref", FAUX sinon
	 */
	protected boolean findRef (String ref) {
		Iterator iter = data.iterator();
		while (iter.hasNext()) {
			ElemSaveObject item = (ElemSaveObject) iter.next();
			if (item.reference!=null) {
				if (item.reference.equals(ref)) return true;
				if (item.value instanceof InputSaveObject)
					if (((InputSaveObject) item.value).findRef(ref)) return true;
			}
		}
		return false;
	}


}
