/*
 * Created on 8 fvr. 2005
 *
 */

package util.saveObject;

/**
 * <p>Tout objet devant tre sauvegard et rcupr par les classes "OutputSaveObject"
 * et "InputSaveObject" doit implmenter cette interface. Les 2 mthodes "saveInto"
 * et "loadFrom" sont en effet appeles lors du processus de sauvegarde.
 * 
 * <p>Le principe est le suivant : lors de la sauvegarde de l'objet, la fonction
 * "saveInto" est appele. L'objet est alors cens stocker dans 'sObj' toutes les donnes
 *  conserver (entier, chaine de caractre, objets 'Savable',...) ncessaires
 * et suffisantes a sa reconstruction. C'est en effet un 'sObj' similaire qui sera 
 * transmis par la fonction 'LoadFrom' et dans lequel l'objet pourra rcuprer toutes les 
 * donnes prcedemment stockes. 
 *   
 * 
 * @author Benjamin LE BOZEC
 */
public interface Savable {

	/**
	 * Cette fonction permet la persistance de l'objet en fournissant  celui-ci
	 * un 'OutputSaveObject' dans lequel il peut enregistr toutes les donnes
	 * ncessaires et suffisantes  sa reconstruction.
	 * 
	 * @param sObj L'objet de stockage (paramtre d'entre / sortie)
	 * @throws SaveObjectException Cette exception peut etre leve par la mthode
	 * 'add' de 'OutputSaveObject' ('add' est en effet cense tre appele  l'interieure
	 * de 'saveInto') 
	 */
	public void saveInto (OutputSaveObject sObj) throws SaveObjectException;

	/**
	 * Cette fonction permet la reconstruction de l'objet en fournissant  celui-ci
	 * un 'InputSaveObject' dans lequel toutes les donnees prcdemment sauves
	 * par la mthode 'saveInto' sont disponibles.
	 * 
	 * @param sObj L'objet de stockage (entre uniquement)
	 * @throws SaveObjectException
	 */
	public void loadFrom (InputSaveObject sObj) 
		throws SaveObjectException;

}
